"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _template = _interopRequireDefault(require("@babel/template"));

var t = _interopRequireWildcard(require("@babel/types"));

var _murmurHash = _interopRequireDefault(require("@adeira/murmur-hash"));

var _sxTailwind = require("@adeira/sx-tailwind");

module.exports = function sxTailwindBabelPlugin() {
  var stylesCollector = [];
  var tailwindCallee = '';
  var sxtCallee = '';
  var stylesVarName = 'styles';
  return {
    name: 'sx-tailwind-babel-transform',
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        if (path.node.source.value === '@adeira/sx-tailwind') {
          path.node.specifiers.forEach(function (_ref) {
            var imported = _ref.imported,
                local = _ref.local;

            if (imported.name === 'tailwind') {
              tailwindCallee = local.name;
            } else if (imported.name === 'sxt') {
              sxtCallee = local.name;
            }
          });

          var sxImport = _template.default.ast("import * as sx from '@adeira/sx'");

          path.replaceWith(sxImport);
        }
      },
      JSXAttribute: function JSXAttribute(path) {
        var _path$node, _path$node$name, _path$node$name2, _path$node$value, _path$node$value$expr, _path$node$value$expr2, _path$node$value$expr3;

        if (((_path$node = path.node) === null || _path$node === void 0 ? void 0 : (_path$node$name = _path$node.name) === null || _path$node$name === void 0 ? void 0 : _path$node$name.type) !== 'JSXIdentifier' || ((_path$node$name2 = path.node.name) === null || _path$node$name2 === void 0 ? void 0 : _path$node$name2.name) !== 'className' || ((_path$node$value = path.node.value) === null || _path$node$value === void 0 ? void 0 : _path$node$value.type) !== 'JSXExpressionContainer' || ((_path$node$value$expr = path.node.value.expression) === null || _path$node$value$expr === void 0 ? void 0 : _path$node$value$expr.type) !== 'CallExpression' || ((_path$node$value$expr2 = path.node.value.expression) === null || _path$node$value$expr2 === void 0 ? void 0 : (_path$node$value$expr3 = _path$node$value$expr2.callee) === null || _path$node$value$expr3 === void 0 ? void 0 : _path$node$value$expr3.type) !== 'Identifier') {
          return;
        }

        if (path.node.value.expression.callee.name === sxtCallee) {
          path.node.value.expression.arguments.forEach(function (a) {
            return stylesCollector.push(a.value);
          });
          path.node.value.expression.callee.name = stylesVarName;
        } else if (path.node.value.expression.callee.name === tailwindCallee) {
          var styles = [];
          path.node.value.expression.arguments[0].value.split(' ').forEach(function (style) {
            stylesCollector.push(style);
            styles.push(style);
          });
          var attribute = t.jsxAttribute(t.jsxIdentifier('className'), t.jsxExpressionContainer(t.callExpression(t.identifier(stylesVarName), styles.map(function (style) {
            return t.stringLiteral(style);
          }))));
          path.replaceWith(attribute);
        }
      },
      Program: {
        enter: function enter(path, state) {
          var filename = state.file.opts.filename.split('/').slice(-2).join('/');
          stylesVarName = "__styles_".concat((0, _murmurHash.default)(filename));
        },
        exit: function exit(path) {
          var declarations = Object.fromEntries(stylesCollector.map(function (style) {
            return [style, _sxTailwind.tailwindStyles[style]];
          }));

          if (Object.keys(declarations).length === 0) {
            return;
          }

          path.node.body.push(_template.default.ast("const ".concat(stylesVarName, " = sx.create(").concat(JSON.stringify(declarations), ")"))); // is there SX imported twice?

          var sxImports = path.node.body.reduce(function (a, _ref2, index) {
            var type = _ref2.type,
                source = _ref2.source,
                specifiers = _ref2.specifiers;

            if (type === 'ImportDeclaration' && source.value === '@adeira/sx' && Array.isArray(specifiers) && specifiers[0].type === 'ImportNamespaceSpecifier') {
              a.push(index);
            }

            return a;
          }, []);

          if (sxImports.length > 1) {
            delete path.node.body[sxImports[0]];
          }
        }
      }
    }
  };
};